import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { DeleteConfigurationCommand } from '../../commands/delete-configuration.command';
import { Configuration } from '../entities/configuration';

@CommandHandler(DeleteConfigurationCommand)
export class DeleteConfigurationUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    private readonly _configurationMessager: ConfigurationMessager,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: DeleteConfigurationCommand): Promise<Configuration> {
    try {
      const configuration = await this._repository.delete(command.uuid);
      this._configurationMessager.publish(
        'delete',
        JSON.stringify({ uuid: configuration.uuid }),
      );
      this._loggingMessager.publish(
        'configuration.delete.info',
        JSON.stringify({ uuid: configuration.uuid }),
      );
      return configuration;
    } catch (error) {
      this._loggingMessager.publish(
        'configuration.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
